@@ -42,6 +42,7 @@ struct state {
 	int err;
 	char *spare_group;
 	int active, working, failed, spare, raid;
+	int from_config;
 	int expected_spares;
 	int devstate[MaxDisks];
 	dev_t devid[MaxDisks];
@@ -69,7 +70,7 @@ static int check_one_sharer(int scan);
 static void alert(char *event, char *dev, char *disc, struct alert_info *info);
 static int check_array(struct state *st, struct mdstat_ent *mdstat,
 		       int test, struct alert_info *info,
-		       int increments);
+		       int increments, char *prefer);
 static int add_new_arrays(struct mdstat_ent *mdstat, struct state **statelist,
 			  int test, struct alert_info *info);
 static void try_spare_migration(struct state *statelist, struct alert_info *info);
@@ -79,7 +80,7 @@ int Monitor(struct mddev_dev *devlist,
 	    char *mailaddr, char *alert_cmd,
 	    int period, int daemonise, int scan, int oneshot,
 	    int dosyslog, int test, char *pidfile, int increments,
-	    int share)
+	    int share, char *prefer)
 {
 	/*
 	 * Every few seconds, scan every md device looking for changes
@@ -124,6 +125,7 @@ int Monitor(struct mddev_dev *devlist,
 	 */
 
 	struct state *statelist = NULL;
+	struct state *st2;
 	int finished = 0;
 	struct mdstat_ent *mdstat = NULL;
 	char *mailfrom = NULL;
@@ -182,7 +184,8 @@ int Monitor(struct mddev_dev *devlist,
 			}
 			st->next = statelist;
 			st->devnum = INT_MAX;
-			st->percent = -2;
+			st->percent = RESYNC_UNKNOWN;
+			st->from_config = 1;
 			st->expected_spares = mdlist->spare_disks;
 			if (mdlist->spare_group)
 				st->spare_group = strdup(mdlist->spare_group);
@@ -198,7 +201,7 @@ int Monitor(struct mddev_dev *devlist,
 			st->devname = strdup(dv->devname);
 			st->next = statelist;
 			st->devnum = INT_MAX;
-			st->percent = -2;
+			st->percent = RESYNC_UNKNOWN;
 			st->expected_spares = -1;
 			if (mdlist) {
 				st->expected_spares = mdlist->spare_disks;
@@ -220,7 +223,8 @@ int Monitor(struct mddev_dev *devlist,
 		mdstat = mdstat_read(oneshot?0:1, 0);
 
 		for (st=statelist; st; st=st->next)
-			if (check_array(st, mdstat, test, &info, increments))
+			if (check_array(st, mdstat, test, &info,
+					increments, prefer))
 				anydegraded = 1;
 		
 		/* now check if there are any new devices found in mdstat */
@@ -242,6 +246,11 @@ int Monitor(struct mddev_dev *devlist,
 		}
 		test = 0;
 	}
+	for (st2 = statelist; st2; st2 = statelist) {
+		statelist = st2->next;
+		free(st2);
+	}
+
 	if (pidfile)
 		unlink(pidfile);
 	return 0;
@@ -288,8 +297,10 @@ static int check_one_sharer(int scan)
 	int pid, rv;
 	FILE *fp;
 	char dir[20];
+	char path[100];
 	struct stat buf;
-	fp = fopen("/var/run/mdadm/autorebuild.pid", "r");
+	sprintf(path, "%s/autorebuild.pid", MDMON_DIR);
+	fp = fopen(path, "r");
 	if (fp) {
 		if (fscanf(fp, "%d", &pid) != 1)
 			pid = -1;
@@ -311,12 +322,12 @@ static int check_one_sharer(int scan)
 		fclose(fp);
 	}
 	if (scan) {
-		if (mkdir("/var/run/mdadm", S_IRWXU) < 0 &&
+		if (mkdir(MDMON_DIR, S_IRWXU) < 0 &&
 		    errno != EEXIST) {
 			fprintf(stderr, Name ": Can't create "
 				"autorebuild.pid file\n");
 		} else {
-			fp = fopen("/var/run/mdadm/autorebuild.pid", "w");
+			fp = fopen(path, "w");
 			if (!fp)
 				fprintf(stderr, Name ": Cannot create"
 					" autorebuild.pid"
@@ -437,7 +448,7 @@ static void alert(char *event, char *dev, char *disc, struct alert_info *info)
 
 static int check_array(struct state *st, struct mdstat_ent *mdstat,
 		       int test, struct alert_info *ainfo,
-		       int increments)
+		       int increments, char *prefer)
 {
 	/* Update the state 'st' to reflect any changes shown in mdstat,
 	 * or found by directly examining the array, and return
@@ -451,6 +462,7 @@ static int check_array(struct state *st, struct mdstat_ent *mdstat,
 	int i;
 	int remaining_disks;
 	int last_disk;
+	int new_array = 0;
 
 	if (test)
 		alert("TestMessage", dev, NULL, ainfo);
@@ -473,7 +485,7 @@ static int check_array(struct state *st, struct mdstat_ent *mdstat,
 	 * have a device disappear than all of them that can
 	 */
 	if (array.level == 0 || array.level == -1) {
-		if (!st->err)
+		if (!st->err && !st->from_config)
 			alert("DeviceDisappeared", dev, "Wrong-Level", ainfo);
 		st->err = 1;
 		close(fd);
@@ -509,6 +521,14 @@ static int check_array(struct state *st, struct mdstat_ent *mdstat,
 		 * just make sure it is always different. */
 		array.utime = st->utime + 1;;
 
+	if (st->err) {
+		/* New array appeared where previously had and error */
+		st->err = 0;
+		st->percent = RESYNC_NONE;
+		new_array = 1;
+		alert("NewArray", st->devname, NULL, ainfo);
+	}
+
 	if (st->utime == array.utime &&
 	    st->failed == array.failed_disks &&
 	    st->working == array.working_disks &&
@@ -517,7 +537,6 @@ static int check_array(struct state *st, struct mdstat_ent *mdstat,
 		    mse->percent == st->percent
 		    ))) {
 		close(fd);
-		st->err = 0;
 		if ((st->active < st->raid) && st->spare == 0)
 			return 1;
 		else
@@ -532,7 +551,7 @@ static int check_array(struct state *st, struct mdstat_ent *mdstat,
 	    st->expected_spares > 0 &&
 	    array.spare_disks < st->expected_spares)
 		alert("SparesMissing", dev, NULL, ainfo);
-	if (st->percent == -1 &&
+	if (st->percent < 0 && st->percent != RESYNC_UNKNOWN &&
 	    mse->percent >= 0)
 		alert("RebuildStarted", dev, NULL, ainfo);
 	if (st->percent >= 0 &&
@@ -548,7 +567,7 @@ static int check_array(struct state *st, struct mdstat_ent *mdstat,
 		alert(percentalert, dev, NULL, ainfo);
 	}
 
-	if (mse->percent == -1 &&
+	if (mse->percent == RESYNC_NONE &&
 	    st->percent >= 0) {
 		/* Rebuild/sync/whatever just finished.
 		 * If there is a number in /mismatch_cnt,
@@ -557,8 +576,10 @@ static int check_array(struct state *st, struct mdstat_ent *mdstat,
 		struct mdinfo *sra =
 			sysfs_read(-1, st->devnum, GET_MISMATCH);
 		if (sra && sra->mismatch_cnt > 0) {
-			char cnt[40];
-			sprintf(cnt, " mismatches found: %d", sra->mismatch_cnt);
+			char cnt[80];
+			snprintf(cnt, sizeof(cnt),
+				 " mismatches found: %d (on raid level %d)",
+				sra->mismatch_cnt, array.level);
 			alert("RebuildFinished", dev, cnt, ainfo);
 		} else
 			alert("RebuildFinished", dev, NULL, ainfo);
@@ -602,45 +623,31 @@ static int check_array(struct state *st, struct mdstat_ent *mdstat,
 		int change;
 		char *dv = NULL;
 		disc.number = i;
-		if (i >= last_disk) {
-			newstate = 0;
-			disc.major = disc.minor = 0;
-		} else if (info[i].major || info[i].minor) {
+		if (i < last_disk &&
+		    (info[i].major || info[i].minor)) {
 			newstate = info[i].state;
-			dv = map_dev(info[i].major, info[i].minor, 1);
+			dv = map_dev_preferred(
+				info[i].major, info[i].minor, 1,
+				prefer);
 			disc.state = newstate;
 			disc.major = info[i].major;
 			disc.minor = info[i].minor;
-		} else if (mse &&  mse->pattern && i < (int)strlen(mse->pattern)) {
-			switch(mse->pattern[i]) {
-			case 'U': newstate = 6 /* ACTIVE/SYNC */; break;
-			case '_': newstate = 8 /* REMOVED */; break;
-			}
-			disc.major = disc.minor = 0;
-		}
+		} else
+			newstate = (1 << MD_DISK_REMOVED);
+
 		if (dv == NULL && st->devid[i])
-			dv = map_dev(major(st->devid[i]),
-				     minor(st->devid[i]), 1);
+			dv = map_dev_preferred(
+				major(st->devid[i]),
+				minor(st->devid[i]), 1, prefer);
 		change = newstate ^ st->devstate[i];
-		if (st->utime && change && !st->err) {
-			if (i < array.raid_disks &&
-			    (((newstate&change)&(1<<MD_DISK_FAULTY)) ||
-			     ((st->devstate[i]&change)&(1<<MD_DISK_ACTIVE)) ||
-			     ((st->devstate[i]&change)&(1<<MD_DISK_SYNC)))
-				)
+		if (st->utime && change && !st->err && !new_array) {
+			if ((st->devstate[i]&change)&(1<<MD_DISK_SYNC))
 				alert("Fail", dev, dv, ainfo);
-			else if (i >= array.raid_disks &&
+			else if ((newstate & (1<<MD_DISK_FAULTY)) &&
 				 (disc.major || disc.minor) &&
-				 st->devid[i] == makedev(disc.major, disc.minor) &&
-				 ((newstate&change)&(1<<MD_DISK_FAULTY))
-				)
+				 st->devid[i] == makedev(disc.major, disc.minor))
 				alert("FailSpare", dev, dv, ainfo);
-			else if (i < array.raid_disks &&
-				 ! (newstate & (1<<MD_DISK_REMOVED)) &&
-				 (((st->devstate[i]&change)&(1<<MD_DISK_FAULTY)) ||
-				  ((newstate&change)&(1<<MD_DISK_ACTIVE)) ||
-				  ((newstate&change)&(1<<MD_DISK_SYNC)))
-				)
+			else if ((newstate&change)&(1<<MD_DISK_SYNC))
 				alert("SpareActive", dev, dv, ainfo);
 		}
 		st->devstate[i] = newstate;
@@ -693,7 +700,7 @@ static int add_new_arrays(struct mdstat_ent *mdstat, struct state **statelist,
 			st->next = *statelist;
 			st->err = 1;
 			st->devnum = mse->devnum;
-			st->percent = -2;
+			st->percent = RESYNC_UNKNOWN;
 			st->expected_spares = -1;
 			if (mse->metadata_version &&
 			    strncmp(mse->metadata_version, "external:", 9) == 0 &&
@@ -705,7 +712,6 @@ static int add_new_arrays(struct mdstat_ent *mdstat, struct state **statelist,
 			*statelist = st;
 			if (test)
 				alert("TestMessage", st->devname, NULL, info);
-			alert("NewArray", st->devname, NULL, info);
 			new_found = 1;
 		}
 	return new_found;
