@@ -468,7 +468,12 @@ static void brief_examine_super1(struct supertype *st, int verbose)
 	else
 		nm = NULL;
 
-	printf("ARRAY%s%s", nm ? " /dev/md/":"", nm);
+	printf("ARRAY ");
+	if (nm) {
+		printf("/dev/md/");
+		print_escape(nm);
+		putchar(' ');
+	}
 	if (verbose && c)
 		printf(" level=%s", c);
 	sb_offset = __le64_to_cpu(sb->super_offset);
@@ -485,8 +490,10 @@ static void brief_examine_super1(struct supertype *st, int verbose)
 		if ((i&3)==0 && i != 0) printf(":");
 		printf("%02x", sb->set_uuid[i]);
 	}
-	if (sb->set_name[0])
-		printf(" name=%.32s", sb->set_name);
+	if (sb->set_name[0]) {
+		printf(" name=");
+		print_quoted(sb->set_name);
+	}
 	printf("\n");
 }
 
@@ -548,8 +555,10 @@ static void brief_detail_super1(struct supertype *st)
 	struct mdp_superblock_1 *sb = st->sb;
 	int i;
 
-	if (sb->set_name[0])
-		printf(" name=%.32s", sb->set_name);
+	if (sb->set_name[0]) {
+		printf(" name=");
+		print_quoted(sb->set_name);
+	}
 	printf(" UUID=");
 	for (i=0; i<16; i++) {
 		if ((i&3)==0 && i != 0) printf(":");
@@ -1096,7 +1105,7 @@ static int write_init_super1(struct supertype *st)
 	unsigned long long dsize, array_size;
 	unsigned long long sb_offset, headroom;
 
-	for (di = st->info; di && ! rv ; di = di->next) {
+	for (di = st->info; di; di = di->next) {
 		if (di->disk.state == 1)
 			continue;
 		if (di->fd < 0)
@@ -1189,16 +1198,17 @@ static int write_init_super1(struct supertype *st)
 		case 1:
 			sb->super_offset = __cpu_to_le64(0);
 			reserved = bm_space + 4*2;
-			/* Try for multiple of 1Meg so it is nicely aligned */
-			#define ONE_MEG (2*1024)
-			reserved = ((reserved + ONE_MEG-1)/ONE_MEG) * ONE_MEG;
-			if (reserved + __le64_to_cpu(sb->size) > dsize)
-				reserved = dsize - __le64_to_cpu(sb->size);
-			/* force 4K alignment */
-			reserved &= ~7ULL;
-
 			if (reserved < headroom)
 				reserved = headroom;
+			if (reserved + array_size > dsize)
+				reserved = dsize - array_size;
+			/* Try for multiple of 1Meg so it is nicely aligned */
+			#define ONE_MEG (2*1024)
+			if (reserved > ONE_MEG)
+				reserved = (reserved/ONE_MEG) * ONE_MEG;
+
+			/* force 4K alignment */
+			reserved &= ~7ULL;
 
 			sb->data_offset = __cpu_to_le64(reserved);
 			sb->data_size = __cpu_to_le64(dsize - reserved);
@@ -1206,22 +1216,23 @@ static int write_init_super1(struct supertype *st)
 		case 2:
 			sb_offset = 4*2;
 			sb->super_offset = __cpu_to_le64(4*2);
-			if (4*2 + 4*2 + bm_space + __le64_to_cpu(sb->size)
+			if (4*2 + 4*2 + bm_space + array_size
 			    > dsize)
-				bm_space = dsize - __le64_to_cpu(sb->size)
+				bm_space = dsize - array_size
 					- 4*2 - 4*2;
 
 			reserved = bm_space + 4*2 + 4*2;
-			/* Try for multiple of 1Meg so it is nicely aligned */
-			#define ONE_MEG (2*1024)
-			reserved = ((reserved + ONE_MEG-1)/ONE_MEG) * ONE_MEG;
-			if (reserved + __le64_to_cpu(sb->size) > dsize)
-				reserved = dsize - __le64_to_cpu(sb->size);
-			/* force 4K alignment */
-			reserved &= ~7ULL;
-
 			if (reserved < headroom)
 				reserved = headroom;
+			if (reserved + array_size > dsize)
+				reserved = dsize - array_size;
+			/* Try for multiple of 1Meg so it is nicely aligned */
+			#define ONE_MEG (2*1024)
+			if (reserved > ONE_MEG)
+				reserved = (reserved/ONE_MEG) * ONE_MEG;
+
+			/* force 4K alignment */
+			reserved &= ~7ULL;
 
 			sb->data_offset = __cpu_to_le64(reserved);
 			sb->data_size = __cpu_to_le64(dsize - reserved);
@@ -1234,13 +1245,14 @@ static int write_init_super1(struct supertype *st)
 			goto out;
 		}
 
-
 		sb->sb_csum = calc_sb_1_csum(sb);
 		rv = store_super1(st, di->fd);
 		if (rv == 0 && (__le32_to_cpu(sb->feature_map) & 1))
 			rv = st->ss->write_bitmap(st, di->fd);
 		close(di->fd);
 		di->fd = -1;
+		if (rv)
+			goto error_out;
 	}
 error_out:
 	if (rv)
